
from dataclasses import field
import sys
import os
import time
import json
import datetime
import shutil

panelPath = '/www/server/panel/'
os.chdir(panelPath)

sys.path.insert(0, panelPath + "class/")
import public
import db


# 数据库配置文件
DATABASES_PATH = os.path.join(public.get_panel_path(), "config/databases.json")

# 每次面板更新，数据库配置文件发生改变后，更新数据库表结构
def update_tb(tb_name: str, tb_info: dict, sqlite_obj: db.Sql):
    field_list = sqlite_obj.query("PRAGMA table_info('{tb_name}');".format(tb_name=tb_name))
    if len(tb_info["fields"]) > len(field_list):  # 表结构不一样，更新
        temp_tb_name = "{}_old_{}".format(tb_name, int(time.time() * 1000_000))

        # 修改表名,将当前表名修改
        sqlite_obj.execute("ALTER TABLE {table_name} RENAME TO {old_table_name};".format(table_name=tb_name, old_table_name=temp_tb_name))
        # 创建表
        resp = sqlite_obj.execute(tb_info["sql"])
        if str(resp).find("error") != -1:
            print("更新失败-创建表 {} 错误：{}".format(tb_name, resp))
            # 创建表错误修改回去
            sqlite_obj.execute("ALTER TABLE {old_table_name} RENAME TO {table_name};".format(old_table_name=temp_tb_name, table_name=tb_name))
        else:
            field_new = {data[1] for data in tb_info["fields"]}
            field_old = {data[1] for data in field_list}
            # 获取并集字段
            field_sql = '"' + '","'.join(field_new & field_old) + '"'

            insert_sql = "INSERT INTO '{tb_name}' ({field_sql}) SELECT {field_sql} FROM '{tb_name_old}';".format(tb_name=tb_name, field_sql=field_sql, tb_name_old=temp_tb_name)
            resp = sqlite_obj.execute(insert_sql)
            if str(resp).find("error") != -1:
                print("更新失败-更新数据 {} 错误：{}".format(tb_name, resp))
                # 插入数据错误还原
                sqlite_obj.execute("DROP TABLE {table_name}".format(table_name=tb_name))
                sqlite_obj.execute("ALTER TABLE {old_table_name} RENAME TO {table_name};".format(old_table_name=temp_tb_name, table_name=tb_name))
            else:  # 成功修复，删除临时表
                sqlite_obj.execute("DROP TABLE {old_table_name}".format(old_table_name=temp_tb_name))
                print("更新成功：{}".format(tb_name))

# 初始化数据库
def init_db(): # 每次重启检查数据库
    try:
        init_db_dict = json.loads(public.readFile(DATABASES_PATH))
    except Exception as err:
        print("读取默认数据库文件错误！{}".format(err))
        return
    db_dir = os.path.join(public.get_panel_path(), "data/db")
    if not os.path.exists(db_dir):
        os.makedirs(db_dir)

    flag_path = os.path.join(db_dir, "update")
    if not os.path.exists(flag_path):
        update_db() #更新
        return

    flag = False
    for db_name, tb_dict in init_db_dict.items():
        db_path = os.path.join(db_dir, db_name)
        sqlite_obj = db.Sql()
        sqlite_obj.dbfile(db_path)
        tb_list = sqlite_obj.query("SELECT name FROM sqlite_master WHERE type='table' AND name not in ('sqlite_sequence','sqlite_master');")
        if not isinstance(tb_list, list):
            print(f"err:{tb_list}")
            continue
        tb_list = [data[0] for data in tb_list]
        for tb_name, tb_info in tb_dict.items():
            if tb_name not in tb_list: # 表不存在
                if flag is False:
                    print("正在初始化面板数据库...")
                    flag = True
                print("正在创建表：{}".format(tb_name))
                resp = sqlite_obj.execute(tb_info["sql"])
                if str(resp).find("error") != -1:
                    print("创建表 {} 错误：{}".format(tb_name, resp))
            else: # 更新表
                update_tb(tb_name, tb_info, sqlite_obj)



def update_db(): # 面板数据库迁移更新 只执行一次
    try:
        init_db_dict = json.loads(public.readFile(DATABASES_PATH))
    except Exception as err:
        print("读取默认数据库文件错误！{}".format(err))
        return
    db_dir = os.path.join(public.get_panel_path(), "data/db")
    if not os.path.exists(db_dir):
        os.makedirs(db_dir)

    flag_path = os.path.join(db_dir, "update")
    is_update = os.path.exists(flag_path)

    default_db_path = os.path.join(public.get_panel_path(), "data/default.db")
    default_sqlite_obj = db.Sql()
    default_sqlite_obj.dbfile(default_db_path)
    shutil.copy(default_db_path, os.path.join(db_dir, "default.db"))

    flag = False
    for db_name, tb_dict in init_db_dict.items():
        if db_name == "docker.db":
            docker_path = os.path.join(public.get_panel_path(), "data/docker.db")
            if os.path.exists(docker_path):
                docker_new_path = os.path.join(db_dir, "docker.db")
                shutil.copy(docker_path, docker_new_path)
            continue
        db_path = os.path.join(db_dir, db_name)
        sqlite_obj = db.Sql()
        sqlite_obj.dbfile(db_path)
        tb_list = sqlite_obj.query("SELECT name FROM sqlite_master WHERE type='table' AND name not in ('sqlite_sequence','sqlite_master');")
        if not isinstance(tb_list, list):
            print(f"err:{tb_list}")
            continue
        tb_list = [data[0] for data in tb_list]
        for tb_name, tb_info in tb_dict.items():
            if tb_name in tb_list:  # 表存在删除
                sqlite_obj.execute("DROP TABLE '{tb_name}';".format(tb_name=tb_name))

            if flag is False:
                print("正在更新面板数据库...")
                flag = True
            print("正在更新表：{}".format(tb_name))

            temp_data = default_sqlite_obj.query("SELECT sql FROM sqlite_master WHERE type='table' AND name='{tb_name}';".format(tb_name=tb_name))
            if not isinstance(temp_data, list) or len(temp_data) == 0:
                continue

            resp = sqlite_obj.execute(temp_data[0][0])
            if str(resp).find("error") != -1:
                print("创建表 {} 错误：{}".format(tb_name, resp))

            # 更新
            if os.path.exists(default_db_path) and not is_update:
                temp_data = default_sqlite_obj.query("SELECT count(*) FROM sqlite_master WHERE type='table' AND name='{tb_name}';".format(tb_name=tb_name))
                if temp_data[0][0] == 0:  # 查询表是否存在
                    continue

                field_list = default_sqlite_obj.query("PRAGMA table_info('{tb_name}');".format(tb_name=tb_name))
                if not isinstance(field_list, list) or len(field_list) == 0:  # 表字段是否为空
                    continue
                field_list = [data[1] for data in field_list]
                field_sql = '"' + '","'.join(field_list) + '"'
                insert_sql = "INSERT INTO '{tb_name}' ({field_sql}) VALUES ({create_sql});".format(tb_name=tb_name, field_sql=field_sql, create_sql=",".join(["?"] * len(field_list)))

                if tb_name == "logs":
                    data_list = default_sqlite_obj.query("SELECT MAX(id) FROM '{tb_name}';".format(tb_name=tb_name))
                    if not isinstance(data_list, list) or len(data_list) == 0:  # 表字段是否为空
                        print(f"log: {data_list}")
                        continue
                    max_id = data_list[0][0]
                    if isinstance(max_id, int): # 日志表插入数据，更新自增id，防止迁移主键冲突
                        sqlite_obj.execute(insert_sql, (max_id +1, "面板更新", "更新数据库", datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"), 1, "www.bt.cn"))

                temp_data = default_sqlite_obj.query("SELECT count(*) FROM {tb_name}".format(tb_name=tb_name))
                if not isinstance(temp_data, list):
                    print(temp_data)
                    continue
                count = temp_data[0][0]
                limit = 1000
                for idx in range(0, count, limit):
                    data_list = default_sqlite_obj.query("SELECT * FROM {tb_name} LIMIT {idx},{limit};".format(tb_name=tb_name, idx=idx, limit=limit))
                    resp = sqlite_obj.executemany(insert_sql, data_list)
                    print(f"row:{resp}")

    public.writeFile(flag_path, "true")
    # public.ExecShell("rm -rf {}".format(db_dir))


#获取数据库对象
def get_db_obj(sfile):
    db_obj = db.Sql()
    db_obj.dbfile(sfile)
    return db_obj

default_db_obj = get_db_obj(os.path.join(panelPath, "data/default.db"))

#default_db_obj = get_db_obj("/www/default.db")

dst_dir = '{}/data/db/'.format(panelPath)
if not os.path.exists(dst_dir): os.makedirs(dst_dir)

#获取表结构
def  get_table_info(db_obj,table):
    field_info  = {}
    wheres = []
    field_keys = []
    tb_list = db_obj.query("PRAGMA table_info({})".format(table))
    if not isinstance(tb_list, list):
        print('{} 表结构读取失败，可能原因表已损坏'.format(table))
        return None

    for tb in tb_list:
        field_info[tb[1]] = tb
        field_keys.append('`{}`'.format(tb[1]))
        wheres.append('`{}`=?'.format(tb[1]))

    res = db_obj.query("SELECT * FROM sqlite_master WHERE type='table' AND name='{}';".format(table))
    if not isinstance(res, list):
        return None

    if len(res) == 0:
        return None
    data = {}
    data['field'] = field_info
    data['sql'] = res[0][4]
    data['field_keys'] = field_keys
    data['wheres'] = wheres
    return data

#检查表是否存在
def check_exists_table(db_obj,table):
    res = db_obj.query("SELECT * FROM sqlite_master WHERE type='table' AND name='{}';".format(table))
    if not isinstance(res, list):
        print('{} 读取失败，可能原因表已损坏'.format(table))
        return False
    if len(res) > 0:
        return True
    return False

#同步单个表
def sync_db_table(table,dst_db_name):
    src_field = get_table_info(default_db_obj,table)
    #检查表存在
    dst_obj = get_db_obj('{}/{}'.format(dst_dir,dst_db_name))
    if not check_exists_table(dst_obj,table):
        print('{}数据库 {} 表不存在，准备创建数据库'.format(dst_db_name,table))
        res = dst_obj.execute(src_field['sql'])
        if str(res).find("error") != -1:
            print("创建表 {} 错误：{}".format(table, res))
            return False

    #修复字段
    dst_field = get_table_info(dst_obj,table)
    for field in src_field['field']:
        if not field in dst_field['field']:
            print('{}数据库 {} 表 {} 字段不存在，准备创建字段'.format(dst_db_name,table,field))
            res = dst_obj.execute('ALTER TABLE {} ADD COLUMN {} {}'.format(table,field,src_field['field'][field][2]))
            if str(res).find("error") != -1:
                print("创建字段 {} 错误：{}".format(field, res))
                return False

    #迁移数据
    sql_field = ','.join(src_field['field_keys']).strip(',')
    resp =  default_db_obj.query("select {} from {};".format(sql_field,table))

    if not isinstance(resp, list):
        print("读取 {} 数据失败：{}".format(table, resp))
        return False

    #过滤where条件，搜索到id字段为止
    s_list = []
    for n_where in src_field['wheres']:
        s_list.append(n_where)
        if n_where.find('id') >= 0: break

    swhere = ' AND '.join(s_list).strip(' AND ')
    for info in resp:
        query_sql = 'SELECT count(*) FROM {} WHERE {};'.format(table,swhere)
        query_res = dst_obj.query(query_sql,info[:len(s_list)])
        if not isinstance(query_res, list):
            print("查询数据失败：{}".format(query_res))
            return False
        if query_res[0][0] > 0:
            continue

        sql = 'INSERT INTO {} ({}) VALUES ({});'.format(table,sql_field,",".join(["?"] * len(info)))
        res = dst_obj.execute(sql,info)
        if str(res).find("error") != -1:
            print(query_res)
            print("插入数据失败：{}".format(res + query_sql + str(info)))
            continue

def sync_db_all():
    flag_path = '{}/update'.format(dst_dir)
    if os.path.exists(flag_path):
        # print('数据库已同步，如需重新同步，删除整个 {} 目录即可'.format(dst_dir))
        return False

    all_tabs = {}
    init_db_dict = json.loads(public.readFile(DATABASES_PATH))
    for db_keys in init_db_dict.keys():
        for table in init_db_dict[db_keys].keys():
            all_tabs[table] = db_keys
    resp = default_db_obj.query("SELECT name FROM sqlite_master WHERE type='table' AND name not in ('sqlite_sequence','sqlite_master');")
    if not isinstance(resp, list):
        print("读取默认数据库失败：{}".format(resp))
        return False

    for table in resp:
        table = table[0]
        if table in all_tabs:
            print('正在同步表到 {} 数据库：{}'.format(table,all_tabs[table]))
            sync_db_table(table,all_tabs[table])
    print('数据库同步完成')
    public.writeFile(flag_path,'true')

# sync_db_all()
#sync_db_table('sites','site.db')
init_db()
